//
// GameX
// Ship Demo - Main Program
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <process.h>

#define GAMEX_MAIN
#include "gamex.hpp"

#include "shipdemox-ship.hpp"				// Include for Ship Class
#include "shipdemox-photons.hpp"			// Include for Photons Class

											//*** Global Variables
Ship ship_red;								// Red ship object
Ship ship_blue;								// Blue ship object
Photons photons;							// Photon object
ImageX back_img;							// Background Image
ImageX rotate_img;							// Rotate Image (multipurpose)
int done;									// Program status (done?)
int frame;									// Current frame

// GameX initialize function (called when program starts)
void initialize (void)
{
	GameX.Initialize (false, 800, 400);		// Initialize GameX itself (required)

	ImageExt imgext;						// Image Extender for loading TIF images

											// Initialize Photons
	photons.Initialize();					
											// Initialize Ships
	ship_red.Initialize(0);	
	ship_blue.Initialize(1);
	ship_red.UsePhotons (&photons);			// Assign Photon Class to Ships to
	ship_blue.UsePhotons (&photons);		// permit ships to 'create' photons when firing

											// Load the background
	imgext.Load ("Images\\ice.tif", &back_img, DRAW_IMAGE);	
	
											// Prepare multi-purpose Rotated Image	
	rotate_img.Size (70, 70, IMG_HIGHCOLOR | IMG_ALPHA);

											// Prepare other global variables
	done=0; frame=0;	
}

// GameX main function (called repeatedly as fast as possible)
void main (void)
{
											// Move Ships & Photons
	ship_red.Move();						// (call respective class member functions)
	ship_blue.Move();
	photons.Move();	

	ship_red.Collisions (&ship_blue);		// Ship-to-Ship Collision Detection
	ship_blue.Collisions (&ship_red);

	// Drawing Begins	
	GameX.DrawOpen ();						// Open GameX for drawing output (required)
											// Draw Background (ice)
		GameX.DrawBackground ((int) (ship_red.GetX()-ship_blue.GetX()), (int) (ship_red.GetY()-ship_blue.GetY()), &back_img);											
		ship_blue.Draw (frame);				// Draw Red and Blue ships
		ship_red.Draw (frame);		
		photons.Draw (frame);				// Draw Photons
		
	GameX.DrawClose ();						// Close GameX output (required)
	// Drawing Ends

	frame++;								// Increment frame number
	if (GameX.GetKey()==27) done=1;			// Check for ESC key
	GameX.ViewUpdate ();					// Update view with newly drawn image (required)
}
